'use strict';

var path = require('path');
var fs = require('fs');
var _extends = require('@babel/runtime/helpers/extends');
var debug = require('debug');
require('lodash.includes');
require('param-case');
var PropTypes = require('prop-types');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var _extends__default = /*#__PURE__*/_interopDefaultLegacy(_extends);
var debug__default = /*#__PURE__*/_interopDefaultLegacy(debug);
var PropTypes__default = /*#__PURE__*/_interopDefaultLegacy(PropTypes);

var name = "tailwind-react-primitives";

var basePath = __dirname.replace(new RegExp(name + "/.*"), name);

var components = fs__default['default'].readdirSync(path__default['default'].resolve(basePath, 'src/components'));
var getPaths = (function () {
  return components.map(function (component) {
    return path__default['default'].join(basePath, "src/components/" + component + "/*.tsx");
  });
});

var display = {
  block: PropTypes__default['default'].bool,
  hidden: PropTypes__default['default'].bool,
  inline: PropTypes__default['default'].bool,
  inlineBlock: PropTypes__default['default'].bool,
  table: PropTypes__default['default'].bool,
  tableCell: PropTypes__default['default'].bool,
  tableRow: PropTypes__default['default'].bool
};
var floats = {
  clearfix: PropTypes__default['default'].bool,
  float: PropTypes__default['default'].oneOf(['none', 'right', 'left'])
};
var overflow = {
  overflow: PropTypes__default['default'].oneOf(['hidden', 'auto', 'scroll']),
  overflowX: PropTypes__default['default'].oneOf(['hidden', 'auto', 'scroll']),
  overflowY: PropTypes__default['default'].oneOf(['hidden', 'auto', 'scroll'])
};
var position = {
  absolute: PropTypes__default['default'].bool,
  fixed: PropTypes__default['default'].bool,
  inset: PropTypes__default['default'].oneOfType([PropTypes__default['default'].oneOf(['0', 'y-0', 'x-0', 'auto']), PropTypes__default['default'].number]),
  top: PropTypes__default['default'].number,
  bottom: PropTypes__default['default'].number,
  left: PropTypes__default['default'].number,
  right: PropTypes__default['default'].number,
  relative: PropTypes__default['default'].bool,
  static: PropTypes__default['default'].bool
};
var zIndex = {
  z: PropTypes__default['default'].oneOfType([PropTypes__default['default'].number, PropTypes__default['default'].string])
};
var typography = {
  align: PropTypes__default['default'].string,
  break: PropTypes__default['default'].oneOf(['words', 'normal']),
  capitalize: PropTypes__default['default'].bool,
  font: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].array]),
  italic: PropTypes__default['default'].bool,
  text: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].array]),
  tracking: PropTypes__default['default'].string,
  leading: PropTypes__default['default'].string,
  lineThrough: PropTypes__default['default'].bool,
  lowercase: PropTypes__default['default'].bool,
  normalCase: PropTypes__default['default'].bool,
  noUnderline: PropTypes__default['default'].bool,
  notItalic: PropTypes__default['default'].bool,
  truncate: PropTypes__default['default'].bool,
  underline: PropTypes__default['default'].bool,
  uppercase: PropTypes__default['default'].bool,
  whitespace: PropTypes__default['default'].oneOf(['normal', 'no-wrap', 'pre', 'pre-line', 'pre-wrap'])
};
var backgrounds = {
  bg: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].array])
};
var borders = {
  border: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].bool, PropTypes__default['default'].array]),
  borderB: PropTypes__default['default'].number,
  borderL: PropTypes__default['default'].number,
  borderR: PropTypes__default['default'].number,
  borderT: PropTypes__default['default'].number,
  rounded: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].bool]),
  roundedB: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].bool]),
  roundedBl: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].bool]),
  roundedBr: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].bool]),
  roundedL: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].bool]),
  roundedR: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].bool]),
  roundedT: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].bool]),
  roundedTl: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].bool]),
  roundedTr: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].bool])
};
var flexValues = [true, 'row', 'row-reverse', 'col', 'col-reverse', 'no-wrap', 'wrap', 'wrap-reverse', 'initial', 1, 'auto', 'none', 'grow', 'shrink', 'grow-0', 'shrink-0'];
var flexAlignment = ['start', 'center', 'end'];
var flex = {
  content: PropTypes__default['default'].oneOf(flexAlignment.concat(['between', 'around'])),
  flex: PropTypes__default['default'].oneOfType([PropTypes__default['default'].oneOf(flexValues), PropTypes__default['default'].arrayOf(PropTypes__default['default'].oneOf(flexValues))]),
  inlineFlex: PropTypes__default['default'].bool,
  items: PropTypes__default['default'].oneOf(flexAlignment.concat(['stretch', 'baseline'])),
  self: PropTypes__default['default'].oneOf(flexAlignment.concat(['auto', 'stretch'])),
  justify: PropTypes__default['default'].oneOf(flexAlignment.concat(['between', 'around']))
};
var spacingShape = {
  t: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].number]),
  r: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].number]),
  b: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].number]),
  l: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].number]),
  x: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].number]),
  y: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].number])
};
var spacing = {
  m: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].number, PropTypes__default['default'].shape(spacingShape)]),
  nm: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].number, PropTypes__default['default'].shape(spacingShape)]),
  p: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].number, PropTypes__default['default'].shape(spacingShape)])
};
var sizing = {
  h: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].number]),
  maxH: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].number]),
  minH: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].number]),
  maxW: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].number]),
  minW: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].number]),
  w: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].number])
};
var misc = {
  container: PropTypes__default['default'].bool,
  opacity: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].number]),
  shadow: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].bool]),
  list: PropTypes__default['default'].oneOf(['none', 'list', 'decimal']),
  outline: PropTypes__default['default'].string,
  appearance: PropTypes__default['default'].string,
  select: PropTypes__default['default'].string,
  pointerEvents: PropTypes__default['default'].string,
  fill: PropTypes__default['default'].string,
  cursor: PropTypes__default['default'].oneOf(['auto', 'default', 'pointer', 'wait', 'text', 'move', 'not-allowed'])
};
var plugins = {
  visuallyHidden: PropTypes__default['default'].bool,
  visuallyHiddenFocusable: PropTypes__default['default'].bool,
  focusable: PropTypes__default['default'].bool
};
var propTypes = _extends__default['default']({}, display, floats, overflow, position, zIndex, typography, backgrounds, borders, flex, spacing, sizing, misc, plugins);
/**
 * Custom Tailwind `variant:` classes that customise when a style is applied
 * TODO: Should handle custom breakpoints
 */

var propVariants = ['hover', 'focus', 'hocus', 'sm', 'md', 'lg', 'xl'];
/**
 *  Custom QoL props that don't follow the normal transformation conventions
 */

var helperPropsMap = {
  'w-auto': ['flex-1'],
  'focusable': ['focus:outline-none', 'focus:shadow-outline'],
  'size': ['text'],
  'color': ['text'],
  'weight': ['font'],
  'bold': ['font-bold']
};
var tailwindProps = Object.keys(propTypes).concat(propVariants.reduce(function (variantProps, variant) {
  return variantProps.concat(Object.keys(propTypes).map(function (prop) {
    return prop + "-" + variant;
  }));
}, []));

var log = debug__default['default']('tailwind-react:extractor');
var propRegex = {
  /** Detects `<Box select="none" />` */
  singleValue: function singleValue(prop) {
    return new RegExp(prop + "(?:-[A-Za-z]+)?=[`\"'{\\s]+[\\w\\/\\-]+\\s*[`\"'}]+", 'g');
  },

  /** Detects `<Box flex />` */
  booleanValue: function booleanValue(prop) {
    return new RegExp("\\s" + prop + "(?:-[A-Za-z]+)?(?:\\s+|>)", 'g');
  },

  /** Detects `<Box cursor={disabled ? 'default' : 'pointer'} />` & `<Box cursor={disabled || 'pointer'} />` */
  conditionalValue: function conditionalValue(prop) {
    return new RegExp(prop + "(?:-[A-Za-z]+)?={[\\w\\s\\?!:\"'`]+(?:\\?|\\|)+[\\w\\s\\?!:\"'`]+}", 'g');
  },

  /** Detects `<Box m={{ x: 'auto' }} />` */
  objectValue: function objectValue(prop) {
    return new RegExp(prop + "(?:-[A-Za-z]+)?={{\\s*[\\w\\s:\\-\\/'\"`,]+\\s*}}", 'g');
  },

  /** Detects `<Box flex={flex={[true, wrap && 'wrap', col ? 'col' : 'row']} />` */
  arrayValue: function arrayValue(prop) {
    return new RegExp(prop + "(?:-[A-Za-z]+)?={\\s*\\[[^\\]]+]", 'g');
  },
  className: /className=[`"'{\s]+[\w\s/-]+\s*[`"'}]+/g
};
var searchTerms = tailwindProps.concat(Object.keys(helperPropsMap));

var transformations = _extends__default['default']({}, helperPropsMap);

var purgeFromTailwindReact = (function (content) {
  var classNames = [];
  log('Starting extract');
  searchTerms.forEach(function (prop) {
    try {
      var matches = []; // Single Value

      var singleValueMatches = content.match(propRegex.singleValue(prop)) || [];
      matches = matches.concat(singleValueMatches);
      if (singleValueMatches.length) log('  - single prop value matches:', singleValueMatches); // Boolean

      var booleanMatches = content.match(propRegex.booleanValue(prop)) || [];
      matches = matches.concat(booleanMatches);

      if (booleanMatches.length) {
        log('  - boolean prop value matches:', booleanMatches);
      } // Objects


      var objMatches = content.match(propRegex.objectValue(prop)) || [];

      if (objMatches.length) {
        log('  - object prop value matches', objMatches);
        matches = matches.concat(objMatches.reduce(function (array, match) {
          return array.concat(match.replace(/\s/g, '').replace(prop + "=", '').replace(/[{}'"]/g, '').split(',').map(function (objProp) {
            var classFragments = objProp.split(':');
            var key = classFragments[0];
            return key.length === 1 ? "" + prop + key + "-" + classFragments[1] : "" + prop + (key === 'def' ? '' : "-" + classFragments[0]) + "-" + classFragments[1];
          }));
        }, []));
      } // Conditionals


      var conditionalMatches = content.match(propRegex.conditionalValue(prop)) || [];

      if (conditionalMatches.length) {
        log('  - conditional prop value matches:', conditionalMatches);
        conditionalMatches.forEach(function (propAndValue) {
          var _a, _b;

          var propName = (_b = (_a = propAndValue.match(/[A-Za-z]+(?==)/g)) === null || _a === void 0 ? void 0 : _a[0]) !== null && _b !== void 0 ? _b : '';
          var conditionalValues = propAndValue.match(/{.+(?=}\s*)/g) || [];

          if (!conditionalValues[0]) {
            matches.push(propName);
            return;
          }

          if (conditionalValues[0].indexOf('?') === -1) {
            conditionalValues = conditionalValues[0].replace(/`'"/, '').split(/\s+\|\|\s+/);
          } else {
            conditionalValues = conditionalValues[0].replace(/{[a-zA-Z]+\s+\?\s+/, '').split(/\s+:\s+/);
          }

          matches = matches.concat([propName].concat(conditionalValues.map(function (value) {
            return propName + "-" + value.replace(/["']/g, '');
          })));
        });
      } // Arrays


      var arrayMatches = content.match(propRegex.arrayValue(prop)) || [];

      if (arrayMatches.length) {
        log('  - array prop value matches:', arrayMatches);
        arrayMatches.forEach(function (propAndValue) {
          var _a, _b;

          var propName = (_b = (_a = propAndValue.match(/[A-Za-z]+(?==)/g)) === null || _a === void 0 ? void 0 : _a[0]) !== null && _b !== void 0 ? _b : '';
          var arrayValues = propAndValue.replace(propName, '');
          matches = matches.concat([propName].concat((arrayValues.match(/[\w-]+/g) || []).map(function (value) {
            return propName + "-" + value;
          })));
        });
      } // Format matches found


      classNames = classNames.concat(matches // Remove defaults
      .filter(function (className, index, array) {
        return array.indexOf(className) === index;
      }) // Remove any special characters, e.g. string wrappers
      .map(function (match) {
        return match.replace(/[\s"'{}>]/g, '');
      }) // Add variant classes
      .reduce(function (classes, match) {
        var className = [match];
        propVariants.forEach(function (variant) {
          if (match.includes("-" + variant)) {
            var propName = match.replace("-" + variant, '');
            className = variant === 'hocus' ? ["hover:" + propName, "focus:" + propName] : [variant + ":" + propName];
          }
        });
        return classes.concat(className);
      }, []) // Transform helper props into underlying tailwind class
      .reduce(function (classes, match) {
        var transformation = transformations[match];

        if (transformation) {
          return classes.concat(transformation);
        }

        var propName = match;

        if (match.indexOf(':') !== -1) {
          var classFragments = match.split(':');
          propName = classFragments[1];
        }

        transformation = transformations[propName];

        if (transformation) {
          return classes.concat(transformation.map(function (t) {
            return match.replace(propName, t);
          }));
        }

        if (match.indexOf('=') !== -1) {
          var _classFragments = match.split('=');

          propName = _classFragments[0];
        }

        transformation = transformations[propName];

        if (transformation) {
          return classes.concat(transformation.map(function (t) {
            return match.replace(propName, t);
          }));
        }

        return classes.concat([match]);
      }, []) // Apply final transformations to mimic tailwind class naming conventions
      .map(function (match) {
        return match.replace('=', '-').replace(/[A-Z]/g, '-$&').toLowerCase();
      }));
    } catch (err) {
      throw err;
    }
  }); // Manual classNames check

  var classNameMatches = content.match(propRegex.className) || [];

  if (classNameMatches.length) {
    log('  - className prop value matches:', classNameMatches);
    classNameMatches.forEach(function (className) {
      classNames = classNames.concat(className.replace('className=', '').match(/[\w-/:]+(?<!:)/g) || []);
    });
  }

  classNames = classNames.filter(function (className, index, array) {
    return array.indexOf(className) === index;
  });

  if (classNames.length) {
    log('Finished and found:', classNames);
  }

  return classNames;
});

exports.getPaths = getPaths;
exports.purgeFromTailwindReact = purgeFromTailwindReact;
//# sourceMappingURL=index.js.map
